#include <stdio.h>
#include <stdlib.h>
#include <math.h>

int rddat (int nmax, int imax, float *g);
int wrrat (int nmax, float *r);
int wreof (int nmax, float *g);
int wrpc (int nmax, int imax, float *g);
int conv1 (int nmax, int imax, float *g, double *gg);
int conv2 (int nmax, int imax, double *gg, float *g);
int norm (int nmax, int imax, float *g);
int cvar (int nmax, int imax, int i1, float *g, float *c);
int score (int nmax, int imax, int i1, float *g, float *p, float *t);
int eigen (int mmax, double *a, double *q, double *r, int lmax);
int qrd (int mmax, double *a, double *q, double *r);
int copy (int mmax, double *a, double *b);
int unit (int mmax, double *a);
int prod (int mmax, double *a, double *b, double *c);
int main (void);

/*========================================*/
/*  Subroutines                           */
/*========================================*/

int rddat (int nmax, int imax, float *g){
  FILE *fpi;
  size_t size;
  char infl[81];
  sprintf (infl, "%s", "input.dat");
  fpi = fopen (infl, "rb");
  size = fread (g, sizeof(float), nmax*imax, fpi);
  fclose (fpi);
  return 0;
}

int wrrat (int nmax, float *r){
  FILE *fpo;
  int n;
  float sum;
  char outfl[81];
  sprintf (outfl, "%s", "ratio.dat");
  fpo = fopen (outfl, "w");
  fprintf (fpo, "%s\n", " No. ratio [%]");
  sum = 0.0;
  for (n=1; n<=nmax; n++){
    sum = sum + r[(n-1)*nmax+n-1];
  }
  for (n=1; n<=nmax; n++){
    fprintf (fpo, "%4d %9.3f\n", n, 100.*r[(n-1)*nmax+n-1]/sum);
  }
  fclose (fpo);
  printf (" %s\n", "Output was written to file 'ratio.dat'.");
  return 0;
}

int wreof (int nmax, float *g){
  FILE *fpo;
  size_t size;
  char outfl[81];
  sprintf (outfl, "%s", "eof.dat");
  fpo = fopen (outfl, "wb");
  size = fwrite (g, sizeof(float), nmax*nmax, fpo);
  fclose (fpo);
  printf (" %s\n", "Output was written to file 'eof.dat'.");
  return 0;
}

int wrpc (int nmax, int imax, float *g){
  FILE *fpo;
  size_t size;
  char outfl[81];
  sprintf (outfl, "%s", "pc.dat");
  fpo = fopen (outfl, "wb");
  size = fwrite (g, sizeof(float), nmax*imax, fpo);
  fclose (fpo);
  printf (" %s\n", "Output was written to file 'pc.dat'.");
  return 0;
}

int conv1 (int nmax, int imax, float *g, double *gg){
  int n, i;
  for (i=1; i<=imax; i++){
    for (n=1; n<=nmax; n++){
      gg[(i-1)*nmax+n-1] = g[(i-1)*nmax+n-1];
    }
  }
  return 0;
}

int conv2 (int nmax, int imax, double *gg, float *g){
  int n, i;
  for (i=1; i<=imax; i++){
    for (n=1; n<=nmax; n++){
      g[(i-1)*nmax+n-1] = gg[(i-1)*nmax+n-1];
    }
  }
  return 0;
}

/*========================================*/

int norm (int nmax, int imax, float *g){
  int n, i;
  float s1, s2, a, d, sd;
  for (n=1; n<=nmax; n++){
    s1 = 0.;
    s2 = 0.;
    for (i=1; i<=imax; i++){
      s1 = s1 + g[(i-1)*nmax+n-1];
      s2 = s2 + g[(i-1)*nmax+n-1] * g[(i-1)*nmax+n-1];
    }
    a = s1 / imax;
    d = s2 / imax - a * a;
    sd = sqrt (d);
    for (i=1; i<=imax; i++){
      g[(i-1)*nmax+n-1] = (g[(i-1)*nmax+n-1] - a) / sd;
    }
  }
  return 0;
}

int cvar (int nmax, int imax, int i1, float *g, float *c){
  int l, m, i;
  float sx, sy, sx2, sy2, sxy, ax, ay;
  for (m=1; m<=nmax; m++){
    for (l=1; l<=m; l++){
      sx = 0.;
      sy = 0.;
      sx2 = 0.;
      sy2 = 0.;
      sxy = 0.;
      for (i=1; i<=i1; i++){
	sx = sx + g[(i-1)*nmax+l-1];
	sy = sy + g[(i-1)*nmax+m-1];
	sx2 = sx2 + g[(i-1)*nmax+l-1] * g[(i-1)*nmax+l-1];
	sy2 = sy2 + g[(i-1)*nmax+m-1] * g[(i-1)*nmax+m-1];
	sxy = sxy + g[(i-1)*nmax+l-1] * g[(i-1)*nmax+m-1];
      }
      ax = sx / i1;
      ay = sy / i1;
      c[(m-1)*nmax+l-1] = (sxy - i1*ax*ay) / i1;
      if (l != m){c[(l-1)*nmax+m-1] = c[(m-1)*nmax+l-1];}
    }
  }
  return 0;
}

int score (int nmax, int imax, int i1, float *g, float *p, float *t){
  int l, n, i;
  for (i=1; i<=i1; i++){
    for (n=1; n<=nmax; n++){
      t[(i-1)*nmax+n-1] = 0.0;
      for (l=1; l<=nmax; l++){
	t[(i-1)*nmax+n-1] = t[(i-1)*nmax+n-1]
	  + p[(n-1)*nmax+l-1] * g[(i-1)*nmax+l-1];
      }
    }
  }
  return 0;
}

/*========================================*/

int eigen (int mmax, double *a, double *q, double *r, int lmax){
  int check;
  int l;
  double *qq, *qaq, *tmp;
  /*
  mmax: Size of the matrix
  a: Input matrix
  q: Orthogonal matrix
  r: Upper triangular matrix
  lmax: Repetition number of QR decomposition
  qq, qaq, tmp: Dummy matrices
  */
  qq = malloc (mmax*mmax*sizeof(double));
  qaq = malloc (mmax*mmax*sizeof(double));
  tmp = malloc (mmax*mmax*sizeof(double));
  check = unit (mmax, qq);
  check = copy (mmax, qaq, a);
  for (l=1; l<=lmax; l++){
    check = qrd (mmax, qaq, q, r);
    check = prod (mmax, qaq, r, q);
    check = copy (mmax, tmp, qq);
    check = prod (mmax, qq, tmp, q);
  }
  check = copy (mmax, q, qq);
  free (qq);
  free (qaq);
  free (tmp);
  return 0;
}

/*========================================*/

int qrd (int mmax, double *a, double *q, double *r){
  int l, m, n, nn;
  /*
  mmax : Size of the matrix
  a: Input matrix
  q: Orthogonal matrix
  r: Upper triangular matrix
  */
  for (n=1; n<=mmax; n++){
    for (nn=1; nn<=n-1; nn++){
      r[(n-1)*mmax+nn-1] = 0.0;
      for (m=1; m<=mmax; m++){
	r[(n-1)*mmax+nn-1] = r[(n-1)*mmax+nn-1]
	  + a[(n-1)*mmax+m-1] * q[(nn-1)*mmax+m-1];
      }
    }
    r[(n-1)*mmax+n-1] = 0.0;
    for (m=1; m<=mmax; m++){
      q[(n-1)*mmax+m-1] = a[(n-1)*mmax+m-1];
      for (nn=1; nn<=n-1; nn++){
	q[(n-1)*mmax+m-1] = q[(n-1)*mmax+m-1]
	  - q[(nn-1)*mmax+m-1] * r[(n-1)*mmax+nn-1];
      }
      r[(n-1)*mmax+n-1] = r[(n-1)*mmax+n-1]
	+ q[(n-1)*mmax+m-1] * q[(n-1)*mmax+m-1];
    }
    r[(n-1)*mmax+n-1] = sqrt(r[(n-1)*mmax+n-1]);
    for (m=1; m<=mmax; m++){
      q[(n-1)*mmax+m-1] = q[(n-1)*mmax+m-1] / r[(n-1)*mmax+n-1];
    }
    for (nn=n+1; nn<=mmax; nn++){
      r[(n-1)*mmax+nn-1] = 0.0;
    }
  }
  return 0;
}

/*========================================*/

int copy (int mmax, double *a, double *b){
  int m, n;
  for (n=1; n<=mmax; n++){
    for (m=1; m<=mmax; m++){
      a[(n-1)*mmax+m-1] = b[(n-1)*mmax+m-1];
    }
  }
  return 0;
}

int unit (int mmax, double *a){
  int m, n;
  for (n=1; n<=mmax; n++){
    for (m=1; m<=mmax; m++){
      if (m == n){
	a[(n-1)*mmax+m-1] = 1.0;
      }else{
	a[(n-1)*mmax+m-1] = 0.0;
      }
    }
  }
  return 0;
}

int prod (int mmax, double *a, double *b, double *c){
  int l, m, n;
  /*
  mmax: Size of the matrix
  a: Product of two matrices (b and c)
  b, c: Input matrix
  */
  for (n=1; n<=mmax; n++){
    for (m=1; m<=mmax; m++){
      a[(n-1)*mmax+m-1] = 0.0;
      for (l=1; l<=mmax; l++){
	a[(n-1)*mmax+m-1] = a[(n-1)*mmax+m-1]
	  + b[(l-1)*mmax+m-1] * c[(n-1)*mmax+l-1];
      }
    }
  }
  return 0;
}

/*========================================*/
/*  Main program                          */
/*========================================*/

int main (){
  size_t size;
  int check;
  int nmax=202, imax=51;
  float *g, *c, *q, *r, *t;
  double *cc, *qq, *rr;

  g = malloc (nmax*imax*sizeof(float));
  c = malloc (nmax*nmax*sizeof(float));
  q = malloc (nmax*nmax*sizeof(float));
  r = malloc (nmax*nmax*sizeof(float));
  t = malloc (nmax*imax*sizeof(float));
  cc = malloc (nmax*nmax*sizeof(double));
  qq = malloc (nmax*nmax*sizeof(double));
  rr = malloc (nmax*nmax*sizeof(double));
  check = rddat (nmax, imax, g);
  check = norm (nmax, imax, g);
  check = cvar (nmax, imax, imax, g, c);
  check = conv1 (nmax, nmax, c, cc);
  check = eigen (nmax, cc, qq, rr, 100);
  check = conv2 (nmax, nmax, qq, q);
  check = conv2 (nmax, nmax, rr, r);
  check = score (nmax, imax, imax, g, q, t);
  check = wrrat (nmax, r);
  check = wreof (nmax, q);
  check = wrpc (nmax, imax, t);
  free (g);
  free (c);
  free (q);
  free (r);
  free (t);
  free (cc);
  free (qq);
  free (rr);
  return 0;
}
