      SUBROUTINE EGNRL (MX,A,E,Q,R,U,LX,QAQ,TMP)
C     MX : Size of the matrix
C     A  : Input matrix
C     E  : Eigen vectors
C     Q  : Orthogonal matrix obtained by QR decomposition
C     R  : Upper triangular matrix obtained by QR decomposition
C          The diagonal components represent eigen values
C     U  : Unit matrix to check validity of calculation
C          Check this matrix whether it is actually a unit matrix.
C          If it is not a unit matrix, it means that the calculation is invalid.
C     LX : Repetition number of QR decomposition
C     QAQ,TMP : Dummy matrices
      REAL*8 A(MX,MX),E(MX,MX),Q(MX,MX),R(MX,MX),U(MX,MX)
      REAL*8 QAQ(MX,MX),TMP(MX,MX)
      CALL EIGEN (MX,A,Q,R,LX,E,QAQ,TMP)
      CALL QRD   (MX,QAQ,U,R)
      CALL QRMOD (MX,U,R)
      CALL EGNUT (MX,QAQ,TMP)
      CALL PROD  (MX,E,Q,TMP)
      RETURN
      END

C ==================================================

      SUBROUTINE EGNUT (MX,A,E)
      REAL*8 A(MX,MX),E(MX,MX),SUM,C
C     MX : Size of the matrix
C     A  : Input matrix (upper traiangle)
C     E  : Eigen vectors
      DO 11 M=1,MX
      DO 12 L=1,MX
        E(L,M) = 0.0D0
   12 CONTINUE
   11 CONTINUE
      DO 21 M=1,MX
        E(M,M) = 1.0D0
      DO 22 L=M-1,1,-1
        SUM = 0.0D0
      DO 23 LL=L+1,M
        SUM = SUM + A(L,LL) * E(LL,M) 
   23 CONTINUE
        E(L,M) = - SUM / (A(L,L)-A(M,M))
   22 CONTINUE
   21 CONTINUE
      DO 31 M=1,MX
        SUM = 0.0D0
      DO 32 L=1,MX
        SUM = SUM + E(L,M)**2
   32 CONTINUE
        C = 1.0D0 / DSQRT(SUM)
      DO 33 L=1,MX
        E(L,M) = C * E(L,M)
   33 CONTINUE
   31 CONTINUE
      RETURN
      END

C ==================================================

      SUBROUTINE EIGEN (MX,A,Q,R,LX,QQ,QAQ,TMP)
      REAL*8 A(MX,MX),Q(MX,MX),R(MX,MX)
      REAL*8 QQ(MX,MX),QAQ(MX,MX),TMP(MX,MX)
C     MX : Size of the matrix
C     A  : Input matrix
C     Q  : Orthogonal matrix
C          This matrix represents eigen vectors of symmetric matrix.
C     R  : Upper triangular matrix
C          The diagonal components represent absolute values of eigen values.
C     LX : Repetition number of QR decomposition
C     QQ,QAQ,TMP : Dummy matrices
      CALL UNIT (MX,QQ)
      CALL COPY (MX,QAQ,A)
      DO 11 L=1,LX
        CALL QRD  (MX,QAQ,Q,R)
        CALL PROD (MX,QAQ,R,Q)
        CALL COPY (MX,TMP,QQ)
        CALL PROD (MX,QQ,TMP,Q)
   11 CONTINUE
      CALL COPY (MX,Q,QQ)
      RETURN
      END

C ==================================================

      SUBROUTINE QRD (MX,A,Q,R)
      REAL*8 A(MX,MX),Q(MX,MX),R(MX,MX)
C     MX : Size of the matrix
C     A  : Input matrix
C     Q  : Orthogonal matrix
C     R  : Upper triangular matrix
      DO 11 N=1,MX
      DO 12 NN=1,N-1
        R(NN,N) = 0.0D0
      DO 13 M=1,MX
        R(NN,N) = R(NN,N) + A(M,N) * Q(M,NN)
   13 CONTINUE
   12 CONTINUE
        R(N,N) = 0.0D0
      DO 14 M=1,MX
        Q(M,N) = A(M,N)
      DO 15 NN=1,N-1
        Q(M,N) = Q(M,N) - Q(M,NN) * R(NN,N)
   15 CONTINUE
        R(N,N) = R(N,N) + Q(M,N)**2
   14 CONTINUE
        R(N,N) = DSQRT(R(N,N))
      DO 16 M=1,MX
        Q(M,N) = Q(M,N) / R(N,N)
   16 CONTINUE
      DO 17 NN=N+1,MX
        R(NN,N) = 0.0D0
   17 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE QRMOD (MX,Q,R)
      REAL*8 Q(MX,MX),R(MX,MX)
C     MX : Size of the matrix
C     Q  : Orthogonal matrix
C     R  : Upper triangular matrix
      DO 11 N=1,MX
      IF (Q(N,N).LT.0.0D0) THEN
      DO 12 M=1,MX
        Q(M,N) = - Q(M,N)
        R(N,M) = - R(N,M)
   12 CONTINUE
      ENDIF
   11 CONTINUE
      RETURN
      END

C ==================================================

      SUBROUTINE COPY (MX,A,B)
      REAL*8 A(MX,MX),B(MX,MX)
      DO 11 N=1,MX
      DO 12 M=1,MX
        A(M,N) = B(M,N)
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE UNIT (MX,A)
      REAL*8 A(MX,MX)
      DO 11 N=1,MX
      DO 12 M=1,MX
      IF (M.EQ.N) THEN
        A(M,N) = 1.0D0
      ELSE
        A(M,N) = 0.0D0
      ENDIF
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE PROD (MX,A,B,C)
      REAL*8 A(MX,MX),B(MX,MX),C(MX,MX)
C     MX  : Size of the matrix
C     A   : Product of two matrices (B and C)
C     B,C : Input matrix
      DO 11 N=1,MX
      DO 12 M=1,MX
        A(M,N) = 0.0D0
      DO 13 L=1,MX
        A(M,N) = A(M,N) + B(M,L) * C(L,N)
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END
