      SUBROUTINE SVD (MX,A,U,S,V,LX,ATA,D1,D2,D3)
      REAL*8 A(MX,MX),U(MX,MX),S(MX,MX),V(MX,MX)
      REAL*8 ATA(MX,MX),D1(MX,MX),D2(MX,MX),D3(MX,MX)
C     MX : Size of the matrix
C     A  : Input matrix
C     U  : Left eigen vectors
C     S  : Diagonal matrix
C          The diagonal components represent singular values.
C     V  : Right eigen vectors
C     LX : Repetition number of QR decomposition
C     ATA,D1,D2,D3 : Dummy matrices
      CALL TRANS  (MX,D1,A)
      CALL PROD   (MX,ATA,A,D1)
      CALL EIGEN  (MX,ATA,U,S,LX,D1,D2,D3)
      DO 11 N=1,MX
      DO 12 M=1,MX
      IF (M.EQ.N) THEN
        S(M,N) = DSQRT(S(M,N))
      ELSE
        S(M,N) = 0.0D0
      ENDIF
   12 CONTINUE
   11 CONTINUE
      CALL PROD   (MX,D1,U,S)
      CALL INVMTX (MX,D1,D2,D3)
      CALL PROD   (MX,D1,D2,A)
      CALL TRANS  (MX,V,D1)
      RETURN
      END

      SUBROUTINE SVDN (MX,NX,A,U,S,V,LX,ATA,D1,D2,D3)
      REAL*8 A(MX,NX),U(MX,MX),S(MX,MX),V(NX,MX)
      REAL*8 ATA(MX,MX),D1(MX,MX),D2(MX,MX),D3(MX,MX)
C     MX, NX : Size of the matrix
C              MX <= NX.
C     A  : Input matrix
C     U  : Left eigen vectors
C     S  : Diagonal matrix
C          The diagonal components represent singular values.
C     V  : Right eigen vectors
C     LX : Repetition number of QR decomposition
C     ATA,D1,D2,D3 : Dummy matrices
      DO 11 M=1,MX
      DO 12 N=1,MX
        ATA(M,N) = 0.0D0
      DO 13 I=1,NX
        ATA(M,N) = ATA(M,N) + A(M,I)*A(N,I)
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      CALL EIGEN  (MX,ATA,U,S,LX,D1,D2,D3)
      DO 21 N=1,MX
      DO 22 M=1,MX
      IF (M.EQ.N) THEN
        S(M,N) = DSQRT(S(M,N))
      ELSE
        S(M,N) = 0.0D0
      ENDIF
   22 CONTINUE
   21 CONTINUE
      CALL PROD   (MX,D1,U,S)
      CALL INVMTX (MX,D1,D2,D3)
      DO 31 M=1,MX
      DO 32 N=1,NX
        V(N,M) = 0.0D0
      DO 33 I=1,MX
        V(N,M) = V(N,M) + D2(M,I)*A(I,N)
   33 CONTINUE
   32 CONTINUE
   31 CONTINUE
      RETURN
      END

      SUBROUTINE SVDM (MX,NX,A,U,S,V,LX,ATA,D1,D2,D3)
      REAL*8 A(MX,NX),U(MX,NX),S(NX,NX),V(NX,NX)
      REAL*8 ATA(NX,NX),D1(NX,NX),D2(NX,NX),D3(NX,NX)
C     MX, NX : Size of the matrix
C              MX >= NX.
C     A  : Input matrix
C     U  : Left eigen vectors
C     S  : Diagonal matrix
C          The diagonal components represent singular values.
C     V  : Right eigen vectors
C     LX : Repetition number of QR decomposition
C     ATA,D1,D2,D3 : Dummy matrices
      DO 11 N=1,NX
      DO 12 M=1,NX
        ATA(M,N) = 0.0D0
      DO 13 I=1,MX
        ATA(M,N) = ATA(M,N) + A(I,M)*A(I,N)
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      CALL EIGEN  (NX,ATA,V,S,LX,D1,D2,D3)
      DO 21 N=1,NX
      DO 22 M=1,NX
      IF (M.EQ.N) THEN
        S(M,N) = DSQRT(S(M,N))
      ELSE
        S(M,N) = 0.0D0
      ENDIF
   22 CONTINUE
   21 CONTINUE
      CALL PROD   (NX,D1,V,S)
      CALL INVMTX (NX,D1,D2,D3)
      DO 31 N=1,NX
      DO 32 M=1,MX
        U(M,N) = 0.0D0
      DO 33 I=1,NX
        U(M,N) = U(M,N) + D2(N,I)*A(M,I)
   33 CONTINUE
   32 CONTINUE
   31 CONTINUE
      RETURN
      END

C ==================================================

      SUBROUTINE EIGEN (MX,A,Q,R,LX,QQ,QAQ,TMP)
      REAL*8 A(MX,MX),Q(MX,MX),R(MX,MX)
      REAL*8 QQ(MX,MX),QAQ(MX,MX),TMP(MX,MX)
C     MX : Size of the matrix
C     A  : Input matrix
C     Q  : Orthogonal matrix
C          This matrix represents eigen vectors of symmetric matrix.
C     R  : Upper triangular matrix
C          The diagonal components represent absolute values of eigen values.
C     LX : Repetition number of QR decomposition
C     QQ,QAQ,TMP : Dummy matrices
      CALL UNIT (MX,QQ)
      CALL COPY (MX,QAQ,A)
      DO 11 L=1,LX
        CALL QRD  (MX,QAQ,Q,R)
        CALL PROD (MX,QAQ,R,Q)
        CALL COPY (MX,TMP,QQ)
        CALL PROD (MX,QQ,TMP,Q)
   11 CONTINUE
      CALL COPY (MX,Q,QQ)
      RETURN
      END

C ==================================================

      SUBROUTINE QRD (MX,A,Q,R)
      REAL*8 A(MX,MX),Q(MX,MX),R(MX,MX)
C     MX : Size of the matrix
C     A  : Input matrix
C     Q  : Orthogonal matrix
C     R  : Upper triangular matrix
      DO 11 N=1,MX
      DO 12 NN=1,N-1
        R(NN,N) = 0.0D0
      DO 13 M=1,MX
        R(NN,N) = R(NN,N) + A(M,N) * Q(M,NN)
   13 CONTINUE
   12 CONTINUE
        R(N,N) = 0.0D0
      DO 14 M=1,MX
        Q(M,N) = A(M,N)
      DO 15 NN=1,N-1
        Q(M,N) = Q(M,N) - Q(M,NN) * R(NN,N)
   15 CONTINUE
        R(N,N) = R(N,N) + Q(M,N)**2
   14 CONTINUE
        R(N,N) = DSQRT(R(N,N))
      DO 16 M=1,MX
        Q(M,N) = Q(M,N) / R(N,N)
   16 CONTINUE
      DO 17 NN=N+1,MX
        R(NN,N) = 0.0D0
   17 CONTINUE
   11 CONTINUE
      RETURN
      END

C ==================================================

      SUBROUTINE INVMTX (IX,A,B,D)
      REAL*8 A(IX,IX),B(IX,IX),D(IX,IX),C
C     IX : Size of the matrix
C     A  : Input matrix
C     B  : Inverse matrix
C     D  : Dummy matrix
      CALL COPY (IX,D,A)
      CALL UNIT (IX,B)
      DO 11 I=1,IX
        C = 1.0D0 / D(I,I)
        CALL ETRL2 (IX,I,C,D)
        CALL ETRL2 (IX,I,C,B)
      DO 12 II=1,IX
      IF (II.NE.I) THEN
        C = - D(II,I)
        CALL ETRL3 (IX,II,I,C,D)
        CALL ETRL3 (IX,II,I,C,B)
      ENDIF
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

C ==================================================

      SUBROUTINE ETRL1 (IX,I1,I2,A)
      REAL*8 A(IX,IX),C
C     Left elementary transformation (1)
C     Change lines.
      DO 11 II=1,IX
        C = A(I1,II)
        A(I1,II) = A(I2,II)
        A(I2,II) = C
   11 CONTINUE
      RETURN
      END

      SUBROUTINE ETRL2 (IX,I,C,A)
      REAL*8 A(IX,IX),C
C     Left elementary transformation (2)
C     Multiply a line by a parameter.
      DO 11 II=1,IX
        A(I,II) = C * A(I,II)
   11 CONTINUE
      RETURN
      END

      SUBROUTINE ETRL3 (IX,I1,I2,C,A)
      REAL*8 A(IX,IX),C
C     Left elementary transformation (3)
C     Add another line multiplied by a parameter to a line.
      DO 11 I=1,IX
        A(I1,I) = A(I1,I) + C * A(I2,I)
   11 CONTINUE
      RETURN
      END

      SUBROUTINE ETRR1 (IX,I1,I2,A)
      REAL*8 A(IX,IX),C
C     Right elementary transformation (1)
C     Change columns.
      DO 11 II=1,IX
        C = A(II,I1)
        A(II,I1) = A(II,I2)
        A(II,I2) = C
   11 CONTINUE
      RETURN
      END

      SUBROUTINE ETRR2 (IX,I,C,A)
      REAL*8 A(IX,IX),C
C     Right elementary transformation (2)
C     Multiply a column by a parameter.
      DO 11 II=1,IX
        A(II,I) = C * A(II,I)
   11 CONTINUE
      RETURN
      END

      SUBROUTINE ETRR3 (IX,I1,I2,C,A)
      REAL*8 A(IX,IX),C
C     Right elementary transformation (3)
C     Add another column multiplied by a parameter to a column.
      DO 11 I=1,IX
        A(I,I1) = A(I,I1) + C * A(I,I2)
   11 CONTINUE
      RETURN
      END

C ==================================================

      SUBROUTINE COPY (MX,A,B)
      REAL*8 A(MX,MX),B(MX,MX)
      DO 11 N=1,MX
      DO 12 M=1,MX
        A(M,N) = B(M,N)
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE UNIT (MX,A)
      REAL*8 A(MX,MX)
      DO 11 N=1,MX
      DO 12 M=1,MX
      IF (M.EQ.N) THEN
        A(M,N) = 1.0D0
      ELSE
        A(M,N) = 0.0D0
      ENDIF
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE PROD (MX,A,B,C)
      REAL*8 A(MX,MX),B(MX,MX),C(MX,MX)
C     MX  : Size of the matrix
C     A   : Product of two matrices (B and C)
C     B,C : Input matrix
      DO 11 N=1,MX
      DO 12 M=1,MX
        A(M,N) = 0.0D0
      DO 13 L=1,MX
        A(M,N) = A(M,N) + B(M,L) * C(L,N)
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE TRANS (MX,A,B)
      REAL*8 A(MX,MX),B(MX,MX)
C     MX : Size of the matrix
C     A  : Output matrix
C     B  : Input matrix
      DO 11 N=1,MX
      DO 12 M=1,MX
        A(M,N) = B(N,M)
   12 CONTINUE
   11 CONTINUE
      RETURN
      END
