      SUBROUTINE EIGEN (MX,A,Q,R,LX,QQ,QAQ,TMP)
      REAL A(MX,MX),Q(MX,MX),R(MX,MX)
      REAL QQ(MX,MX),QAQ(MX,MX),TMP(MX,MX)
C     MX : Size of the matrix
C     A  : Input matrix
C     Q  : Orthogonal matrix
C          This matrix represents eigen vectors of symmetric matrix.
C     R  : Upper triangular matrix
C          The diagonal components represent absolute values of eigen values.
C     LX : Repetition number of QR decomposition
C     QQ,QAQ,TMP : Dummy matrices
      CALL UNIT (MX,QQ)
      CALL COPY (MX,QAQ,A)
      DO 11 L=1,LX
        CALL QRD  (MX,QAQ,Q,R)
        CALL PROD (MX,QAQ,R,Q)
        CALL COPY (MX,TMP,QQ)
        CALL PROD (MX,QQ,TMP,Q)
   11 CONTINUE
      CALL COPY (MX,Q,QQ)
      RETURN
      END

C ==================================================

      SUBROUTINE QRD (MX,A,Q,R)
      REAL A(MX,MX),Q(MX,MX),R(MX,MX)
C     MX : Size of the matrix
C     A  : Input matrix
C     Q  : Orthogonal matrix
C     R  : Upper triangular matrix
      DO 11 N=1,MX
      DO 12 NN=1,N-1
        R(NN,N) = 0.0E0
      DO 13 M=1,MX
        R(NN,N) = R(NN,N) + A(M,N) * Q(M,NN)
   13 CONTINUE
   12 CONTINUE
        R(N,N) = 0.0E0
      DO 14 M=1,MX
        Q(M,N) = A(M,N)
      DO 15 NN=1,N-1
        Q(M,N) = Q(M,N) - Q(M,NN) * R(NN,N)
   15 CONTINUE
        R(N,N) = R(N,N) + Q(M,N)**2
   14 CONTINUE
        R(N,N) = SQRT(R(N,N))
      DO 16 M=1,MX
        Q(M,N) = Q(M,N) / R(N,N)
   16 CONTINUE
      DO 17 NN=N+1,MX
        R(NN,N) = 0.0E0
   17 CONTINUE
   11 CONTINUE
      RETURN
      END

C ==================================================

      SUBROUTINE COPY (MX,A,B)
      REAL A(MX,MX),B(MX,MX)
      DO 11 N=1,MX
      DO 12 M=1,MX
        A(M,N) = B(M,N)
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE UNIT (MX,A)
      REAL A(MX,MX)
      DO 11 N=1,MX
      DO 12 M=1,MX
      IF (M.EQ.N) THEN
        A(M,N) = 1.0E0
      ELSE
        A(M,N) = 0.0E0
      ENDIF
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE PROD (MX,A,B,C)
      REAL A(MX,MX),B(MX,MX),C(MX,MX)
C     MX  : Size of the matrix
C     A   : Product of two matrices (B and C)
C     B,C : Input matrix
      DO 11 N=1,MX
      DO 12 M=1,MX
        A(M,N) = 0.0E0
      DO 13 L=1,MX
        A(M,N) = A(M,N) + B(M,L) * C(L,N)
   13 CONTINUE
   12 CONTINUE
   11 CONTINUE
      RETURN
      END
